@@ -52,6 +52,10 @@
 
 /*
  * change log:
+ #
+ * April 10, 1996
+ * - bounce any message for which the Sender field contains
+ *   mailer-daemon@
  *
  * September 28, 1995 by Keith Moore
  * - allow setting of max_domains from the command-line
@@ -87,13 +91,16 @@
 #include <sysexits.h>
 #include <errno.h>
 #include <time.h>
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
 #include "patchlevel.h"
 
 #ifndef PIPECOMMAND
 #define PIPECOMMAND "/usr/lib/sendmail -bs %s"
 #endif
 
-#ifndef HAS_STRERROR
+#ifndef HAVE_STRERROR
 extern int sys_nerr;
 extern char *sys_errlist[];
 #define strerror(e) ((e) < 0 || (e) >= sys_nerr ? "(unknown)" : sys_errlist[e])
@@ -104,11 +111,13 @@ extern char *sys_errlist[];
 extern int errno;
 #endif
 
-#ifndef HAS_STRDUP
+#ifndef HAVE_STRDUP
 char *strdup();
 #endif
+#ifndef STDC_HEADERS
 char *strchr();
 char *strrchr();
+#endif
 char *malloc ();
 char *realloc ();
 char *mktemp ();
@@ -121,6 +130,7 @@ char *mktemp ();
  */
 #define MAX_DOMAINS 20
 
+char *comment = NULL;		/* generate comment hdr with this value */
 int debug_flag = 0;		/* don't mail; spit SMTP to stdout */
 char *envelope_from;		/* outgoing SMTP from address */
 int force_reply_to = 0;		/* override sender's reply-to hdr */
@@ -131,6 +141,7 @@ int max_size = 0x7fffffff;	/* max message size before bouncing */
 char *precedence = NULL;	/* generate precedence hdr with this value */
 char *reply_to = NULL;		/* generate reply-to hdr with this value  */
 char *sendmail_flags = "";	/* call sendmail with these flags */
+int subscribe_flag = 0;		/* check for "[un]subscribe" in body */
 int verbose_flag = 0;		/* be verbose */
 
 /*
@@ -234,7 +245,7 @@ char *domain;
     return strupper (result);
 }
 
-#ifndef HAS_STRDUP
+#ifndef HAVE_STRDUP
 /*
  * copy a string into malloc'ed memory
  * (most systems have this built-in, but not ultrix 4.3.)
@@ -262,6 +273,8 @@ char *str;
 
 static char *
 strndup (str, len)
+char *str;
+int len;
 {
     char *result = malloc_or_else (len + 1);
 
@@ -963,6 +976,10 @@ FILE *out, *in;
 	    }
 	    else if (match_hdr ("content-length", linebuf))
 		goto delete_header;
+	    else if (match_hdr ("disposition-notification-to", linebuf)) {
+		/* XXX should optionally issue an MDN */
+		goto delete_header;
+	    }
 	    else if (match_hdr ("errors-to", linebuf))
 		goto delete_header;
 	    else if (match_hdr ("message-id", linebuf))
@@ -990,6 +1007,15 @@ FILE *out, *in;
 	    }
 	    else if (match_hdr ("return-receipt-to", linebuf))
 		goto delete_header;
+	    else if (match_hdr ("sender", linebuf)) {
+		char *colon = strchr (linebuf, ':');
+
+		if (contains (colon+1, "mailer-daemon@")) {
+		    fprintf (stderr, 
+			     "bulk_mailer: refusing to forward bounced mail to the list\n");
+		    exit (EX_DATAERR);
+		}
+	    }
 	    else if (match_hdr ("subject", linebuf)) {
 		char *colon = strchr (linebuf, ':');
 
@@ -1051,6 +1077,8 @@ FILE *out, *in;
 
     if (precedence != NULL)
 	fprintf (out, "Precedence: %s\n", precedence);
+    if (comment != NULL)
+	fprintf (out, "Comment: %s\n", comment);
 
     /* If we haven't seen a To, Cc, or Bcc Field, add a dummy one. */
     if (has_to_hdr == 0)
@@ -1077,7 +1105,7 @@ FILE *out, *in;
      */
     lines = 0;
     while (fgets (linebuf, sizeof (linebuf), in) != NULL) {
-	if (lines < 5 &&
+	if (lines < 5 && 
 	    /*
 	     * these often occur in English-text unsubscribe requests
 	     */
@@ -1097,7 +1125,8 @@ FILE *out, *in;
      */
     if (has_valid_approved_hdr == 0 && looks_like_bounce)
 	return LOOKS_LIKE_BOUNCE;
-    else if (has_valid_approved_hdr == 0 && saw_command > 0 && lines <= 7)
+    else if (has_valid_approved_hdr == 0 && saw_command > 0 && 
+	     subscribe_flag && lines <= 7)
 	return HAS_EMBEDDED_COMMAND;
     else
 	return MESSAGE_OKAY;
@@ -1190,7 +1219,12 @@ char *argv[];
     char buf[1024];
 
     while (argc > 1 && (*argv[1] == '-' || *argv[1] == '+')) {
-	if (strcmp (argv[1], "-debug") == 0)
+	if (strcmp (argv[1], "-comment") == 0 && argc > 2) {
+	    comment = argv[2];
+	    --argc;
+	    ++argv;
+	}
+	else if (strcmp (argv[1], "-debug") == 0)
 	    ++debug_flag;
 	else if (strcmp (argv[1], "-domain") == 0 && argc > 2) {
 	    if (strchr (argv[2], '.') == NULL) {
@@ -1244,6 +1278,8 @@ char *argv[];
 	    --argc;
 	    ++argv;
 	}
+	else if (strcmp (argv[1], "-s") == 0)
+	    subscribe_flag = 1;
 	else if (strcmp (argv[1], "-sendmail") == 0 && argc > 2) {
 	    sendmail_flags = argv[2];
 	    --argc;
@@ -1264,6 +1300,7 @@ char *argv[];
 	fprintf (stderr, "(message appears on stdin)\n");
 	fprintf (stderr, "options are (NOTE: NR = NOT RECOMMENDED):\n");
 	fprintf (stderr, "\
+    -comment text          add a header of the form \"Comment: text\"\n\
     -debug                 display SMTP output on stderr; don't send msg\n\
     -domain xx.yy.zz       set domain name to 'xx.yy.zz' (overrides default)\n\
     -maxdomains ###        set max # of distinct domains per envelope\n\
@@ -1272,6 +1309,8 @@ char *argv[];
     -precedence keyword    add a \"Precedence: keyword\" header field (NR)\n\
     -reply-to user@domain  add a reply-to header if there's not one there\n\
     +reply-to user@domain  force reply-to header even if there already (NR)\n\
+    -s                     check for 'unsubscribe' & similar commands\n\
+                           (if found: don't deliver, forward to list owner)\n\
     -sendmail flags        pass 'flags' to sendmail\n\
     -v                     turn on verbose mode\n");
 	exit (EX_USAGE);
